"""Define CMFNotification patches for CMFCore, CMFDefault, DCWorkflow
and Archetypes to call appropriated handlers when specific events
occur.

$Id: patches.py 33881 2006-11-18 15:42:29Z dbaty $
"""

from Products.CMFCore.utils import getToolByName

from NotificationTool import ID, LOG


######### CMFDefault.DublinCore patch ###########################
def notifyModified(self):
    """
    Take appropriate action after the resource has been modified

    Update creators and modification_date and call CMFNotification
    tool handler.
    """
    self._orig_notifyModified()
    ntool = getToolByName(self, ID, None)
    if ntool is not None:
        if self.meta_type == 'Discussion Item':
            ## When a discussion item is added, it is reindexed and,
            ## therefore, notifyModified() is called, leading
            ## there. But we want to call a specific handler for this
            ## event.
            ntool.onDiscussionItemCreation(self)
        else:
            ntool.onItemModification(self)

from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
DefaultDublinCoreImpl._orig_notifyModified = DefaultDublinCoreImpl.notifyModified
DefaultDublinCoreImpl.notifyModified = notifyModified
LOG.info('Monkey-patched CMFDefault')
######### End of CMFDefault.DublinCore patch ####################



######### DCWorkflow patch ######################################
def notifySuccess(self, ob, action, result):
    '''
    Notifies this workflow that an action has taken place.
    '''
    self._orig_notifySuccess(ob, action, result)
    ntool = getToolByName(self, ID, None)
    if ntool is not None:
        ntool.onWorkflowTransition(self, ob, action, result)
    pass

from Products.DCWorkflow.DCWorkflow import DCWorkflowDefinition
DCWorkflowDefinition._orig_notifySuccess = DCWorkflowDefinition.notifySuccess
DCWorkflowDefinition.notifySuccess = notifySuccess
LOG.info('Monkey-patched DCWorkflow')
######### End of DCWorkflow patch ###############################



######### CMFCore.RegistrationTool patch ########################
def afterAdd(self, member, id, password, properties):
    """
    Call CMFNotification handler
    """
    self._orig_afterAdd(member, id, password, properties)
    ntool = getToolByName(self, ID, None)
    if ntool is not None:
        ntool.onMemberRegistration(member, properties)

from Products.CMFCore.RegistrationTool import RegistrationTool
RegistrationTool._orig_afterAdd = RegistrationTool.afterAdd
RegistrationTool.afterAdd = afterAdd
######### End of CMFCore.RegistrationTool patch #################



######### CMFCore.MemberDataTool patch ##########################
def notifyMemberModified(self):
    self._orig_notifyModified()
    ntool = getToolByName(self, ID, None)
    if ntool is not None:
        membership = getToolByName(self, 'portal_membership')
        member = membership.getMemberById(self.getId())
        ntool.onMemberModification(member)

from Products.CMFCore.MemberDataTool import MemberData
MemberData._orig_notifyModified = MemberData.notifyModified
MemberData.notifyModified = notifyMemberModified
LOG.info('Monkey-patched CMFCore')
######### End of CMFCore.MemberDataTool patch ###################



######### Archetypes patch ######################################
##
## We could also have patched 'ExtensibleMetadata.notifyModified()'
## which is the Archetypes version of the CMFCore original method.
##
## However, this does not work very well for standard Plone. Actually,
## items in a standard Plone implement an auto-renaming feature (based
## on the title), which is called after 'notifyModified()'. Therefore,
## if we trigger 'onItemModification' on 'notifyModified()', we do not
## yet know the final name (id) of the object.
##
## Fortunately, Archetypes provides two hooks where we can plug
## CMFNotification: 'at_post_create_script' and 'at_post_edit_script'.
##
def at_post_create_script(self):
    self._orig_at_post_create_script()
    ntool = getToolByName(self, ID, None)
    if ntool is not None:
        ntool.onItemCreation(self)

def at_post_edit_script(self):
    self._orig_at_post_edit_script()
    ntool = getToolByName(self, ID, None)
    if ntool is not None:
        ntool.onItemModification(self)

from Products.Archetypes.BaseObject import BaseObject

BaseObject._orig_at_post_create_script = BaseObject.at_post_create_script
BaseObject._orig_at_post_edit_script = BaseObject.at_post_edit_script
BaseObject.at_post_create_script = at_post_create_script
BaseObject.at_post_edit_script = at_post_edit_script
LOG.info('Monkey-patched Archetypes')
######### End of Archetypes patch ###############################
