"""
Miscellaneous tests for CMFNotification

$Id: testMisc.py 23118 2006-05-07 00:22:38Z dbaty $
"""

import os, sys
if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

from Products.CMFCore.utils import getToolByName
from Products.CMFNotification.NotificationTool import ID as NTOOL_ID
from Products.CMFNotification.tests.utils import CMFNotificationTestCase

class TestMisc(CMFNotificationTestCase):
    """Test various utility methods"""

    def afterSetUp(self):
        """
        Called before each tests
        """
        self.createTestUsersAndContent()


    def testGetPath(self):
        """
        Test 'NotificationTool._getPath()'
        """
        portal = self.portal
        ntool = getToolByName(portal, NTOOL_ID)

        getPath = ntool._getPath
        for obj, path in (
            (portal, '/'),
            (portal.folder, '/folder/'),
            (portal.folder.document1, '/folder/document1'),
            (portal.folder.subfolder, '/folder/subfolder/'),
            (portal.folder.subfolder.document2, '/folder/subfolder/document2'),
            ):
            self.failUnless(getPath(obj) == path,
                            'got %s, expected %s' % (getPath(obj),
                                                     path))

    def testGetParents(self):
        """
        Test 'NotificationTool._getParents()'
        """
        ntool = getToolByName(self.portal, NTOOL_ID)

        getParents = ntool._getParents
        for path, parents in (
            ('/', []),
            ('/folder/', ('/', )),
            ('/folder/document1', ('/folder/', '/')),
            ('/folder/subfolder/', ('/folder/', '/')),
            ('/folder/subfolder/document2',
             ('/folder/subfolder/', '/folder/', '/')),
            ):
            self.failUnless(getParents(path) == parents,
                            "getParents('%s') returned %s, "\
                            "expected %s" % (path,
                                             getParents(path),
                                             parents))

if  __name__ == '__main__':
    framework()

def test_suite():
    from unittest import TestSuite, makeSuite
    suite = TestSuite()
    suite.addTest(makeSuite(TestMisc))
    return suite
