"""Define a base class and prepare ZopeTestCase to be used in
CMFNotification tests

Vaguely based on RichDocument tests.

$Id: utils.py 33180 2006-11-09 16:02:11Z dbaty $
"""

## Import the base test case classes
from Testing import ZopeTestCase
from Products.PloneTestCase import PloneTestCase

## Make ZopeTestCase aware of CMFNotification
ZopeTestCase.installProduct('CMFNotification')

## Set up a Plone site
PRODUCTS = ('CMFNotification', )
PloneTestCase.setupPloneSite(products=PRODUCTS)
PloneTestCase.installProduct('Five')


## Import statements
from OFS.SimpleItem import SimpleItem
from Products.CMFCore.utils import getToolByName
from Products.CMFNotification import NotificationTool

## Monkey-patch NotificationTool logger so that it prints to the
## standard output. If we do not do that, the logger does not log
## anything.
## FIXME: I do not know if this bug/feature is expected. (Damien)
if False: ## Enable only when adding new tests and/or debugging.
    import sys
    import traceback

    def log(msg, *args, **kwargs):
        print msg % args
        if kwargs.get('exc_info', False):
            print traceback.print_exception(*sys.exc_info())

    from Products.CMFNotification.NotificationTool import LOG
    LOG.error = log
    LOG.warning = log


## A fake mail host which does not send emails but keep them in a
## list.
NotificationTool.MAIL_HOST_META_TYPES = ('FakeMailHost', )

class FakeMailHost(SimpleItem):
    meta_type = 'FakeMailHost'

    def __init__(self, *args, **kwargs):
        SimpleItem.__init__(self, *args, **kwargs)
        self.sent = []

    def send(self, message):
        self.sent.append(message)

    def getSentList(self):
        return self.sent

    def clearSentList(self):
        self.sent = []


class CMFNotificationTestCase(PloneTestCase.PloneTestCase):
    """
    Base class for CMFNotification tests
    """
    def createTestUsersAndContent(self):
        """
        Create some test users and content

        Three users are created:
        - a manager: 'manager'
        - two members: 'member1' and 'member2'

        The following items are created:
        + folder
        |--+-- document1
        |--+-- subfolder
        |-----+-- document2
        """
        ## Create users
        mtool = getToolByName(self.portal, 'portal_membership')
        users = ({'id': 'member1', 'roles': ('Member', )},
                 {'id': 'member2', 'roles': ('Member', )},
                 {'id': 'manager', 'roles': ('Manager', 'Member')})
        for user in users:
            email = user['id'] + '@example.com'
            mtool.addMember(user['id'], user['id'], user['roles'], [],
                            properties={'email': email})

        ## Create content
        self.login('manager')
        self.portal.invokeFactory('Folder', 'folder')
        folder = self.portal.folder
        folder.invokeFactory('Document', 'document1')
        folder.invokeFactory('Folder', 'subfolder')
        subfolder = folder.subfolder
        subfolder.invokeFactory('Document', 'document2')
